#!/usr/bin/ruby
# Below please find some examples of how to use the Direct Mail API from Ruby. 
# 
# Before trying these examples, please make sure and fill in your own API Key ID and Key Secret, as well as the
# Form ID. For more information on authentication and authorization, please see
# 
# https://www.directmailmac.com/support/article/349-authentication-and-authorization

require 'net/http'
require 'net/https'
require 'uri'
require 'rubygems'
require 'json'

$API_KEY_ID     = 'replace-me'
$API_KEY_SECRET = 'replace-me'
$FORM_ID        = 'replace-me'

def execute_api_requests( method, url, body = nil)
    # Setup HTTPS Object
    uri = URI.parse( url )
    http = Net::HTTP.new( uri.host, uri.port )
    http.use_ssl = true
    http.verify_mode = OpenSSL::SSL::VERIFY_NONE
    
    # Setup request with appropriate message  
    request = Net::HTTP.const_get( method.capitalize ).new( uri.request_uri )
    request.basic_auth( $API_KEY_ID, $API_KEY_SECRET )
    response = nil
    
    if body
        request.body = JSON.generate( body )
    end

    begin
        response = http.request( request ).body
    rescue Exception => e
        puts "Caught exception while requesting %s %s: %s" % [ method, url, e ]
    end

    if response.to_s.strip.length > 0
        response = JSON.parse( response )
    end

    return response
end

# Example 1: Inserting or updating one address into your mailing list

email   = 'user@example.com'
url     = 'https://secure.directmailmac.com/api/v1/forms/' + $FORM_ID + '/addresses/' + URI::encode( email )
body    = { "create_if_necessary" => true,
            "address" => { "first_name" => "Mike",
                          "last_name" => "Jones",
                          "company_name" => "ABC Widgets, Inc.", 
                          "custom" => { "1" => "650-555-3423" }
                        }
          }

puts execute_api_requests( 'PUT', url, body ).inspect

# Example 2: Deleting one address from your mailing list

email   = 'user@example.com'
url     = 'https://secure.directmailmac.com/api/v1/forms/' + $FORM_ID + '/addresses/' + URI::encode( email ) 

puts execute_api_requests( 'DELETE', url, body ).inspect

# Example 3: Inserting or updating multiple addresses into your mailing list

url     = 'https://secure.directmailmac.com/api/v1/forms/' + $FORM_ID + '/addresses'
body    = { "user1@example.com" => { "create_if_necessary" => true, 
                                    "address" => { "first_name" => "Mike", 
                                                  "last_name" => "Jones",
                                                  "company_name" => "ABC Widgets, Inc.",
                                                  "custom" => { "1" => "650-555-3232" }
                                                }
                                  },
            "user2@example.com" => { "create_if_necessary" => true, 
                                    "address" => { "first_name" => "Mike", 
                                                  "last_name" => "Jones",
                                                  "company_name" => "ABC Widgets, Inc.",
                                                  "custom" => { "1" => "650-555-4321" } 
                                                }
                                  },    
            "user3@example.com" => { "create_if_necessary" => false, 
                                    "address" => { "first_name" => "Rod", 
                                                  "last_name" => "Green",
                                                  "email" => "user3-new@example.com",
                                                  "custom" => { "1" => nil }
                                                }
                                  },                     
          }

puts execute_api_requests( 'PUT', url, body ).inspect

# Example 4: Deleting multiple addresses from your mailing list

url     = 'https://secure.directmailmac.com/api/v1/forms/' + $FORM_ID + '/addresses'
body    = [ "user1@example.com", "user2@example.com", "user3@example.com" ]

puts execute_api_requests( 'DELETE', url, body ).inspect

# Example 5: Marking an address as unsubscribed

email   = 'user@example.com'
url     = 'https://secure.directmailmac.com/api/v1/forms/' + $FORM_ID + '/addresses/' + URI::encode( email )
body    = { "create_if_necessary" => false,
            "address" => { "flags" => { "+" => "unsubscribed" } }
          }
          
puts execute_api_requests( 'PUT', url, body ).inspect